/* vi:ts=4:sw=4:tw=78:fo=croqtl:nocin:
 *
 * VIM - Vi IMproved		by Bram Moolenaar
 *
 * Do ":help uganda"  in Vim to read copying and usage conditions.
 * Do ":help credits" in Vim to see a list of people who contributed.
 */

/*

 Vim originated from Stevie version 3.6 (Fish disk 217) by GRWalter (Fred)

 All the remarks about older versions have been removed, they are not very
 interesting.  Differences between version 3.0 and 4.0 can be found in
 "../doc/vim_40.txt".

  Vim 3.18 (finished 1996 January 30)

--- New ---

Added "-i viminfo" command line option: Set name for .viminfo file. When name
is "NONE" no .viminfo file is read or written.

Added "gI": start insert in column 1.

Made "K" work for visually highlighted text.

Added 'cinwords' option: Keywords that start an extra indent for
'smartindent' and 'cindent' mode.  This used to be a table in the C code, now
it is configurable.  The defaults are the same, should make no difference
unless you change the 'cinw' option.

g<*Mouse> now behaves like <C-*Mouse>.  g<LeftMouse> jumps to tag under
cursor.  <C-RightMouse> and g<RightMouse> do CTRL-T.  Finally you can use the
mouse in an xterm for jumping around in the help files!

Added 'h' flag to 'mouse' option: Use the mouse in a help file.  This allows
for xterm copy/paste in normal files and use the mouse for tag jumps in help
files.

Added file marks to viminfo.  Use '0 to jump back to the file and postition
where the last edit stopped, '1 to older edit, etc.  Now you can quit vim and
restart at the same position later.

When using CTRL-N and CTRL-P in Insert mode, don't use the dictionary when all
matches in the current file have been used.  This could unexpectedly result in
a very slow dictonary search.  You have to use CTRL-X CTRL-K for dictionary
completion.  Define DICT_AFTER_CTRL_N in feature.h if you want the old
behaviour.

--- Incompatible ---

Changed "\w", "\W" and "\p" on the command line to use CTRL-V instead of
backslash.  The backslash may be part of an MSDOS filename.

--- GUI ---

- Adjusted size of scrollbar bubble to remain the same all the time and show
  "~" lines after the end of the file.  Could not get to the last line
  directly when the last screen contained wrapped lines.
- handling of screen resizing could cause an endless loop.
- When starting GUI switch the mouse off for the term where Vim was started.
- Adjusted gui_get_winsize() to get the real size of the window.  Required
  when the geometry is given from app-defaults. Also reset the scroll region.
- When starting a child, set environment variables COLUMNS and LINES, makes
  ":!ls" use the correct number of columns.
- Fixed setenv(), some machines don't have it.  Added our own putenv() for
  when there is none (borrowed from screen).
- Made Backspace and Delete really work in shell.
- Updated the pseudo-tty code.  Copied pty.c from screen.  When opening a pty
  fails, use pipes.  Added 'pty' option, when not set don't use pty. Default
  is to use pipes, since it seems to work on all systems, pty's sometimes
  hang.  Echo characters sent to a pipe.

--- Cindent ---

- Try to get the indent from the previous few lines.  Makes 'cindent' work
  better with code that doesn't match the current 'cinoptions'.  Should make
  Robert Webb very happy!
- A switch() label is put at the same indent of a previous switch() label, if
  one is found.
- Added indent setting '=' for statements after a switch() label. What is a
  better character to use here?
- Unterminated lines are indented the same as previous unterminated lines.
- Don't do cindenting when the indent in the line has been changed manually
  (hit <BS>, <Tab> or <Space> in indent, use CTRL-T or CTRL-D).
- Don't consider a key in CTRL-X mode for triggering cindenting.  Fixes CTRL-X
  CTRL-F causing re-indenting.
- The default values for indents are now 'shiftwidth' instead of 4.
- Various improvements for specific situations.
- Only recognize ':' for a label when it's not followed by another ':',
  in C++ "::" is something completely different.
- Recognize "//" as the start of a comment-line.
- Ignore a match for '{' (start of a block) in a comment.

--- MSDOS & WIN32 ---

- MSDOS: Mouse is on by default.
- MSDOS: Mouse dragging now also supported!
- Don't remove a backslash before a normal file name character. This is not vi
  compatible, but otherwise "normal" MS-DOS file names cannot be used.
Included patches from George Reilly:
- Fixed :recover (sometimes ?) not working.
- I've provided a makefile for the doc directory and various patches.
- The Win32 makefiles now build the executables in the current
  directory for compatibility with all the other makefiles.
- I've changed ExpandWildCards in both win32.c and msdos.c
  so that it doesn't throw away everything if one of the patterns
  fails to match.  This fixes the problem with :recover not working.
- I fixed a fatal bug where :new didn't work (a NULL filename
  was being dereferenced).
- I changed things to use _MAX_PATH from <stdlib.h> instead
  of MAX_PATH because I didn't want to include <windows.h>
  anywhere but win32.c and I had taken MAX_PATH's value
  (260) and put it into win32.h, which I thought dubious practice.

--- Other Improvements ---

When there is an error in a regular expression would get two error messages.
The last one, "invalid search string" is now omitted.  Don't get the "hit
return to continue" message.

Used autoconf 2.7 instead of 2.4 for creating configure.  Doesn't seem to
solve any of our problems though.

For "[i" command and the others that skip comments, also skip a match after a
start-of-comment "/ *" or "/ /".

'smartindent' is now included by default, it's useful for non-C code and C
code that doesn't confirm to some standard that is supported by 'cindent'.

Added setting of HELPDIR by configure.  On BSD 4.4 systems and derivates the
help files should be put in $PREFIX/share/vim.  If the "share" directory
exists it is used, otherwise "lib" is used as before.

Added NO_XMKMF to Makefile, use it if your xmkmf does not work together with
configure (symptoms: X library and/or include directories are messed up).

Put "zsh" together with "ksh" and "sh", instead of with "csh".

":set wc" now outputs "wildchar=<Tab>" instead of "wildchar=9".

--- Fixes ---

When quickfix does not find a complete match, reset the line number to 0,
avoid a jump to a line that could have been any number in the output of the
'makeprg'.

When catching a deadly signal, write the message after resetting the terminal
mode, making it appear on the "normal" screen if there are two screens.

Adjusted digraphs for HPUX, "pp" was used twice.

Moved most comments from version.c to vim_40.txt, removing those that don't
apply to version 3.0.

Fixed ":marks" command: Showed the message twice for normal marks; ":marks c"
also showed generic marks, not just "c"; 'q' at --more-- prompt not handled.
When no (matching) marks are set, give warning message.

Vi compatibility fix: Error message when trying "1,10w", now it is "Use ! to
write partial buffer".  It is also given when the current file does not exist
yet.

Fixed screen jumping when using CTRL-E repetively at the end of the file and
when using GUI scrollbar.

GUI: Insert <S-Space> when typed SHIFT-SPACE in insert mode when preceded with
CTRL-V.

Fixed expanding blockwise visual selection with the right mouse button: would
switch back to characterwise visual mode.

Ignore mouse drag and release events if the click was never seen by the mouse
code.  This makes drag and release be ignored when the click was outside of
the window or when the click has been mapped.  Fixes getting garbage in the
GUI version when mouse click in menu bar (but not on a menu) and dragging it
down into the text.

In insert mode CTRL-V CTRL-@ didn't insert anything, now it inserts ^@ again.
In command line mode CTRL-@ was inserting "<Nul>", this is not compatible with
version 3.0 and it's not very useful.

Made searching a bit more vi-compatible. Fixed search for "/.*p" advancing
only one character at a time instead of jumping to after the 'p'.
"abababababab" only gets three matches for "/abab" instead of five.

When there is no file name and executing a filter command, don't use the file
name of the temporary file for the current file.

Fixed operator with "}", when end is on last line in the file and it's empty.
The last line was included instead of excluded.

Fixed bug: 'writedel' was an int, should be a long.

Fixed bug:  When ":g/pat/d" resulted in an empty file, undo only put one line
back!  Could happen with many other operations that result in an empty file.

Insert a backslash before '~', '.' and '$' when used in a help command.  Makes
":help ~", ":help :." and ":help :$" work.


  Vim 3.19 (finished 1996 February 4)

Unix:
- Changed autoconf check for rename(), NeXTstep cc didn't like the "char" type
  used for it.
- Moved the definition of SIGRETURN from unix.h to config.h, with a check in
  configure.  Fixes NeXTstep cpp not being able to handle
  "#if (RETSIGTYPE != void)".
- Changed exit status when shell cannot be executed from 127 to 122, some
  shells (bash) are already using 127.  Makes error message "cannot execute
  shell" disappear when in fact the command could not be executed, e.g. for
  ":!burp".

GUI:
- moved directory for GUI libraries to before X libraries dir, some systems
  have a libXm in both and we need to use the one from the GUI lib dir.
- Removed thumb and arrows from command line scrollbar, now it's just a box
  that indicates the height of the command line.
- Reduced the maximal value of the horizontal scrollbar, when in the last
  position the screen would jump to the middle.
- Fixed garbage being displayed in the left upper corner for a moment.  The
  cursor was drawn before the screen had been cleared.

WIN32:
- When scrolling back more than 20 lines, clear the screen.  Fixes redraw
  problem after ":e foo" and then CTRL-D 30 times in a 50 lines console
  window. This is not a good solution, just a work around.
- Fixed file name completion after a path name with a space.
- Visual bell: Wait for 15 msec, didnt' see anything on a fast system.
- Removed a newline from call_shell, was not needed.

'cindent':
- Inside a comment, without leading '*', the line after an empty line was put
  in column 0.  Now use the indent of the previous non-empty line.
- When a continuation line is more than one line from the opening paren, use
  the indent of a previous non-empty line.
- "o" on last line of file would scroll screen two lines instead of one.
  replaced cursupdate() by curs_columns() in fixthisline().
- Quite a few other small improvements.

When formatting with "Q" while 'tw' and 'wm' are both zero, use a textwidth of
79, or the screen width minus one if that is smaller.  Joining all the lines
doesn't make sense for the "Q" command.

Fixed vi incompatibility: "o<Esc>u" didn't put the cursor back where it was
before. "O<Esc>u" is still incompatible, this is considered a bug in Vi,
because it puts the cursor in another line.

Fixed: ":e +command filename" would crash.

For Amiga, MS-DOS and Win32: Consider spaces to be part of the file name when
doing completion for a command with a single file argument.

Fixed problem with "gl" when 'wrap' off:  Would scroll the screen towards the
middle if the last character in the line is non printable and it would be
scrolled off the screen.

Don't display 'showbreak' when the line just fits in the window.

When completing with CTRL-L and there are multiple matches, beep!  Ignore case
when comparing file names for MS-DOS and WIN32.  For Amiga don't ignore case
for non-file names.

Renamed regcomp() to vim_regcomp(), regexec() to vim_regexec(), regsub() to
vim_regsug().  Some systems have these as library functions, the prototypes
may be mixed up or the linker would be confused.

Fixed CTRL-X CTRL-L in insert mode giving "pattern not found" twice.

Tag guessing: Allow spaces between function name and '('.

Fixed tack stack: When tag has been found in tags file and Vim jumps to
another file but can't find the line with the tag, CTRL-T didn't work to get
back.

Fixed "J" on last line on the screen not scrolling the text up if the line
doesn't fit in the window anymore.


  Vim 3.20 (finished 1996 February 15)

Implemented events for autocommands.  Only a few are really used now, but
setting and showing them is implemented.  INCOMPATIBLE.

Changed 'langmap' option to a list of pairs.  Now it is possible to translate
any character, not just above 128.  INCOMPATIBLE.

Made ":checkpath!" list ALL included files.

Implemented multi mouse clicks for MSDOS and Unix xterm.  Added 'mousetime'
option.

Completion of 'wildchar' now also show special key name.
Accept any character after ":set wc=", also non-digits. ^I for TAB doesn't
work though, unless you type it as two characters: '^' and 'I'.

Fixed *BUG*: checked for HAVE_LIMIT_H instead of HAVE_LIMTS_H, caused MAXCOL
to be defined to 32767 instead of 2147483647!!!  Now changed definition for
MAXLNUM and MAXCOL to use SIZEOF_INT.  Removed include for limits.h.

Unix:
- Check for a terminating signal while giving an error message for a
  terminating signal (sigh).
- Use mode_t for open() and chmode(), don't check for SCO define (there
  already was an autoconf check for this).
- Included an autoconf check for outfuntype, only use it when it is defined in
  termcap.h.
- Fixed autoconf problems for Ultrix.  Put quotes around xmkmf() and parens
  around the eval for ctags.
- "make install" also makes a link from gvim to vim, when GUI is enabled.
- When CFLAGS is empty, use -O2 for gcc and -O otherwise.  Fixes getting an
  unoptimized executable by default.
- Changed the check for a <sys/wait.h> that defines union wait.  The old check
  was to simple, it just checked for a sys/wait.h that exists but isn't POSIX
  compatible.  Now there is a check if union wait is really defined.
- When swap file contains file name that starts with "~user", it can be
  expanded to "/usr/home/user" or "/home/user".  This makes the "ATTENTION"
  message sometimes not be given when it should.  Now use the inode to check,
  this also works when links are involved and the chance of getting a message
  when the files are different (device is not checked) is very small.  Don't
  use the inode number from the file when it was edited, use the current inode
  number (when inode number has been changed we still want to detect the swap
  file). 

GUI:
- fixed crash for Alpha when using menus.
- Removed inclusion of <stdio.h> and <unistd.h> from gui.c and gui_motif.c,
  these are already included from unix.h.
- Fixed getting <C-CR> when typing CTRL-V CTRL-M.

Win32:
- Fixed bug when scrolling.
- The system and hidden attributes were not being preserved.

MSDOS:
- Don't show mouse cursor unless the mouse is moved.
- Fixed the method to recognize if a mouse is present, for DJGPP it always
  found a mouse, also when there is none, causing random garbage when starting
  up.
- Allow spaces in file names, just like in the win32 version.
- Fixed "vim /autoexec.bat" not working, getting full path for a file name
  starting with "/" was wrong.
- When using the mouse, set area for the mouse, fixes mouse not being able to
  go beyond the 25th line.

'cindent':
- Allow fraction for 'cino': "cino={0.5s".
- 'e' only triggers indent when it's really the second 'e' of an "else" at
  the start of a line.
- recognize nested switches when indenting a case label.
- Recognize that a '{' at the end of a line that starts with a case label.
- When lining up with the character after a '(', skip white space.

Added "(paste)" to show mode in insert mode when paste option is set.

Fixed a few undefined symbols when not defining INSERT_EXPAND and/or AUTOCMD.

Fixed 'showmatch' for "r" command.

Fixed <Home> and <End> in insert mode not setting column to stay in when doing
<Up> or <Down>.

Fixed "M" "zz" and far jump not positioning the window at the same position.
Fixed "zt" and "b" making the screen jump when 'so' is non-zero.
Fixed "M" counting buffer lines instead of screen lines.

Added tag for vim_tags in vim_help.txt, it is sometimes useful to look in the
index of tags for help files.

Improved the :version message.  Options are listed alphabetical.  Added
'langmap'.  Adjust output to the width of the screen.

Renamed 'writedel' to 'writedelay' in the docs.  Made 'writedelay' work for
MS-DOS and Win32.

Fixed setting indent to zero when doing formatting on a long line and
'formatoptions' contains '2'.

Fixed line being printed when there is a '|' after a ":s" command, e.g.
":%s/a/b/g|%s/b/c/g" printed the last line where an 'a' is replaced by a 'b'.

Fixed error message sometimes disappearing when screen is redrawn (e.g. for
"vG:w file" when file exists).

When jumping to a tag without matching case, give a message about it.

Fixed obscure problem: Full file name could become the user name when
'autowrite' is on.

"--more--" prompt:
- Fixed getting a "--more--" prompt before "hit return to continue" without
  anything displayed in between.
- Fixed crash when resizing the window while in the "--more--" prompt in rare
  occasions.
- The character typed at the "--more--" prompt is now directly obtained from
  the user.  This avoids eating mapped characters and syncing undo.
  Disadvantage: Special keys and mouse cannot be used, typeahead is ignored.

When '%', '#', CTRL-Vw, etc. are used on the command line, don't expand
wildcards.  Don't expand wildcards for file name from "gf" and "CTRL-W f"
command.

Fixed chartab not being initialized when doing ":cf fname" from a help file
(and probably in many other situations).  Caused "*" using non-id characters.

Fixed "I" on last line in the window causing a scroll up if the cursor was at
least 'tabstop' columns in the line.

Don't use 'smartcase' for search with "#" or "*" command.

Don't translate key codes in mappings and abbreviations if there are at least
two CTRL-Vs before it (used to be exactly two CTRL-Vs).  Fixes
":cabb hs %s/.^V^V^V^H//g" using "<BS>" instead of CTRL-H.

When reading a file, also set the no-end-of-line when not in binary mode.
When 'binary' is set later and the file is written, the end-of-line is not
written.  Helps when having an autocommand to gunzip "*.gz" files.

When resizing the window and 'equalalways' is set, make all windows of equal
height (all lines were taken/given from/to the last window).

Fixed printing a number option value like it is done for 'wildchar' if the
value is equal to 'wildchar', e.g. when ":set ts=9" it would show
"tabstop=<Tab>".

Fixed cursor positioned on the wrong row (was noticed in the GUI version when
a single line didn't fit in the window, the row of "@@@" was written over the
status line).

Fixed named file marks not working for file where editing started.  Would get
"mark not set" error.

'shellredir' option: allow "%s" where the file name is to be inserted.  Makes
it possible to use "> %s 2>&1", because "2>&1 >" didn't work.

  Vim 3.21 (finished 1996 March 5)

GUI:
- Fixed using a menu causing a crash.
- Fixed ":set ch=2" in .vimrc causing a crash.
- Fixed getting garbage on screen when resizing the window at the "--more--"
  prompt.
- Fixed crash when making the window smaller at the "--more--" prompt.
- use 'mousetime' instead of fixed 250 msec time for multi-mouse-clicks.
- When expanding wildcards, don't send typed characters to the child.
- When the GUI is active, don't set t_kb and t_kD, these are not used.
- Moved updating the menus from when something is written to the screen to
  when a character is asked from the user.  Fixes menus flashing while a
  mapping is being executed.
- When resizing the window at the hit-return prompt, no redraw was done when
  making the window bigger.
- When typing CTRL-Z start a new shell, instead of suspending the process.

MS-DOS:
- Strip the DJGPP binary, makes it a lot smaller.
- DJGPP version now supports long file names, when used with Windows 95.
- Fixed ":w file", when there is no file name for the current buffer, not
  writing the file, and making a backup of the swap file (OOF!).

WIN32:
- Fixed handling of dead keys for Windows NT (Windows 95 has bugs that prevent
  it to work there).
- Fixed "!}fmt" causing a hit-return message, call_shell() didn't return the
  correct value.

Amiga:
- SAS C can't handle ~@ character in a comment (digraph.c).
- Manx C can't handle [0-9] in a scanf() argument, replaced the default for
  'errorformat' with [0123456789].
- Added underline to builtin termcap.

Unix:
- Allow reading from fifos (named pipes) and sockets.  Give a message when
  doing it.
- Include compile command in ":version" message.  Now you can see if the
  optimizer was switched on, which include files have been used, etc.
- Check for memmove() before bcopy(), makes memmove() used when both are
  present, fixes problem with incompatibility between Solaris 2.4 and 2.5.
- When exiting because of a signal, only say "preserving files..." when there
  are really some files to be preserved, not when all .swp files have been
  deleted.
- When preserving files, don't quit when a character is typed.
- For "make shadow": Don't link the doc dir if it already exists.
- Removed "readlenOK" from mch_inchar(), was causing a slowdown on some
  machines, and on machines that do return one char at a time it would be
  eventually set anyway.
- Added gettimeofday() to osdef2.h.in, there was no prototype for this
  function, causing a warning with some compilers.
- Also check 'term' for builtin_iris-ansi when checking for iris-ansi.
- 'ttyfast' is now also on by default for hpterm.
- Ignore case when checking 'term' for being xterm, iris-ansi or hpterm.
- The ATTENTION message was given when using one dir for all swap files,
  already having an edit session for a new "Makefile" and starting an edit for
  "Makefile" in another directory.  Now compare the files names when one of
  the edited files does not exist.

Autocommands:
- The commands for BufNewFIle, BufRead/BufReadPost, BufWritePost,
  FileAppendPost and VimLeave don't change the "buffer changed" flag.  Now
  it's possible to exit after BufRead (decompressed) or BufWritePost (undo)
  changed the buffer.
- The BufWritePost commands are also executed when exiting.
- Added events FileReadPre, FileReadPost, BufReadPre, BufNewFile,
  FileAppendPre, FileAppendPost, FileWritePre, FileWritePost.
- Always set '[ and '] marks after reading a file and before writing a file.
- Support ^Vf on command line for filename of read/written file.
- avoid hit-return message when doing several shell commands.

Mouse:
- Made count argument to do_mouse() a long, caused problems on machines where
  sizeof(int) != sizeof(long).
- Removed setting of 'mouse' to "on", it was too ugly.  Now use "a".
  INCOMPATIBLE with version 3.20.
- When clicking with the mouse causes the screen to scroll, a click in the
  same position is not recognized as a multi-click anymore (except in the
  GUI).  Caused start of Visual mode when quickly clicking near the top/bottom
  of the window when 'so' non-zero.
- Fixed: Mapping of mouse codes after another char didn't work, for example
  ":map g<LeftMouse> <C-RightMouse>".
- Ignore mouse drag and release events after <C-LeftMouse>.  Would start
  Visual mode unexpectedly.

Help:
- For ":help ?<CR>" and ":help :?" don't use the "?" as a wildcard.
- 'helpheight' is only used to set the initial help window height, not when
  entering it later.
- When trying to quit, ignore help windows when deciding to quit Vim.  Now
  when there is only one window and a help window, and we ":q" in the non-help
  window, Vim is exited.

Smarttab:
- When inserting a <Tab> with 'smarttab' set, only round to a multiple of
  'shiftwidth' when 'shiftround' is on.
- Fixed positioning of cursor when inserting a <Tab> with 'smarttab' on and
  'list' on.
- Fixed inserting a <Tab> with 'smarttab' on and 'expandtab' on, wasn't
  correct in all situations.

Viminfo:
- Don't store '[ and '] marks in .viminfo, it's not very useful and makes
  other marks being removed.
- Added " mark: Last known cursor position in a buffer.  Also stored in
  .viminfo file.
- Fixed: ":5d" and ":>" and ":<" didn't set current context mark.
- Fixed: Marks from viminfo file would be deleted for buffers that are in the
  buffer list but have never been loaded.

Fixed making too many columns for ":set" (e.g. when Columns is 96 made five
columns instead of 4).

Added "CTRL-W ^" command, same as "CTRL-W CTRL-^".

Changed default for 'define' and 'include' to use "\t" instead of real <Tab>
(like it's said in the documentation).
 
Added a few checks for a valid screen to avoid potential crashes when starting
up or when resizing the window.

Removed CTRL-B from insert mode, it's confusing.  Added TOGGLE_REVINS define
in feature.h.

Made "zz" at end of file also put cursor in middle of window, making some "~"
lines visible.

Replace environment variables anywhwere in a file name, not just at the
beginning.  Works for options and many other places.

Fixed: ":fixdel" didn't call gather_termleader(), causing the new value of
<Del> sometimes not to be recognized.

Implemented modifiers for '%' and '#' on the command line, "%:p", "%:e", etc.

When replacing '%', '#', etc. on the command line, do wildcard expansion if
there were any wildcards before replacing.  Makes ":so `macros_file %` work.
But for ":e %" no expansion is done, in case the current file name contains a
wildcard.

Made CTRL-U on first line, and CTRL-D on last line in buffer produce a beep
and don't do anything.  This is vi compatible.

Fixed "w" on last char in the file (not being an empty line) not beeping.

In vim_rename() first delete the "to" file.  This makes writing the .viminfo
file work on Window 95, and probably on a few other systems (Xenix?).

Fixed files being written when 'autowrite' is on and a filter command is used.
Also Fixes "WARNING: file changed" message for "ZZ" with gzip-autocommands.

Added "tools/mvi", "tools/fold", "macros/shell.gui" and "macros/html".
Removed "macros/center" and "macros/keyword", they are not very useful
anymore.

Fixed cursor not being positioned correctly, when resizing the window in
Normal mode and the cursor is on a TAB.

For ":checkpath!", when a file is encountered again, don't recurse into it,
but display "(Already listed)" after the name.  Reduces the length of the
output.

When calling a shell, use hit-return message when t_ti is not empty, instead
of checking t_ke.

Added 's' and 'S' flags to 'cpoptions', influence the behaviour of options for
a buffer: Copy when created, copy when first entered or copy every time.
INCOMPATILE with version 3.20, but compatible with version 3.0 (by default)
and vi (when 'S' included).

Changed message "Errorfile not found" to "No errors".

Changed format of 'langmap'.  Allow "abc;ABC" form.  It's upwards compatible.
Fixed ":set langmap=zyZYyzYZ" not working for "Z", because it was mapped to
"Y", which is an acronym for "yy", which was then mapped to "zz".

Remove the character(s) that stopped recording from the register (normally
'q', but can be more when using a mapping for 'q').

Fixed: ">>" shifted lines starting with '#', even when 'cin' set and 'cinkeys'
contains '#' (but 'si' not set).

When 'bin' option is set, save the values of the options that are set to 0.
Restore the values when 'bin is reset.

Added 'ttyscroll' option: Maximum number of screen lines to do scroll with.
When trying to scroll more lines redraw the window.

Fixed: When 'autoindent' not set, formatting text did autoindenting anyway.

For commands like "[i" that search for an identifier in (included) files,
don't consider a line starting with "# define" a comment.  Makes "[i" on
"FALSE" find "# define FALSE 0" instead of some other non-interesting line.

Give beep when trying to use "I" after an operator, without first moving the
cursor to the start of the line.

Change the cursor positioning for CTRL-U and CTRL-D when lines wrap.  The
cursor is now kept a fixed number of FILE lines from the start of the window,
instead of SCREEN lines.  This should make CTRL-U followed by CTRL-D make the
cursor return to the same line in most cases (but not always).


  Vim 3.22 (finished 1996 March 24)

Cindent:
- When inserting text with 'cindent' on, ignore a '(' that is after the
  cursor.  This helps when inserting a new argument before a '('.
- Typing ':' only causes re-indenting if it is after a label.
- Allow spaces between "default" and ":" for a case label.
- ":set cino={s" means to use one shiftwidth for '{', not zero.
- when 'equalprg' is empty, always use cindent, never use ":!indent".
- Don't line up comment lines with a paren.
- Fixed not redrawing when "*^[" included in 'cinkeys' and hitting ESC.
- Adjust start of insert position when changing indent.  Was not possible to
  backspace over inserted characters when typing a label ('bs' = 1).
- Consider a line ending in ',' terminated.  Makes indenting of variable
  initializations a lot better and it's not often used in normal statements.
- A line starting with '{' after an unterminated line is lined up with that
  line.  Makes initializations look better.
- Fixed indent after if () split over more than one line, was looking for last
  ')' in the wrong line.
- When not in any {}, using the indent of a previous line that contains a ')',
  search for the matching paren and match with that line.
- Only recognize a label when the previous line is terminated.  Helps for 
	"var = cond ? \n val1: val2;".
- Allow spaces between name and ':' for a label.

GUI (mostly by Robert Webb and Bill Foster):
- When using a menu, re-sync undo.  All subsequent menu actions would be
  undone with one "u".
- Implemented ":unmenu" and friends.
- Show menus with ":menu", can be further specified.
- Completion of menus for ":menu" command
- When hitting CTRL-Z iconify the window.
- Include patches from Bill Foster:
  - visual bell
  - adjust for border around the text area
  - allow the border to be set
- When closing the window, preserve files and exit.  Vim would hang around and
  needed to be killed.
- Removed inclusion of "config.h" in pty.c.  It's already included from
  "vim.h".  Fixes SVR4 being defined again.
- Implemented "-geometry" command line option and "-scrollbarwidth" command
  line and resource option (Kjetil Jacobsen).
- When window has been scrolled between two mouse clicks, don't recognize it
  as a double click.
- When starting the GUI, avoid opening the X11 window to check for being able
  to restore the title and icon.  Saves some time when starting up.
- Fixed a memory leak: overwriting x11_Display when starting the GUI.

Unix:
- When getting a triple signal (phew!) produce a core dump when needed.
- Changed the check for different file names, would get the ATTENTION message
  when editing files with the same tail on different machines, 'dir' not
  starting with ".".  Also when one of the files being edited doesn't exist.
- Made the emergency exit for an X error the same as for a deadly signal.
- Replaced "ln -sf" with "ln -s" in Makefile, a few systems can't handle "-sf".
- Use getpwnam() to expand ~user when possible.  Should be faster than calling
  the shell and avoids problems with shells that don't understand ~user (some
  versions of /bin/sh)
- Added autoconf check for <pwd.h>, getpwuid() and getpwnam().
- Added autoconf check if sys/time.h and sys/select.h can both be included.
  For some SCO system.
- Motif is checked for in /usr/local/X11* /lib and /usr/local/lib/X11* (added
  the star, matches X11, X11R5 or X11R6).
- Made default for 'dir' ".,~/tmp,/tmp".  Now use ~/tmp when it's available.
  Swap files in /tmp will be gone after rebooting.

MS-DOS:
- Don't set the window size after executing a shell command when executing the
  VimLeave autocommands (would cause the cursor to be positioned anywhere on
  the screen).
- DJGPP: set file name case, like with Amiga.
- Allow spaces in a file name.  Now only Unix doesn't allow this.
- When $VIM is not defined, use $HOME.
- Fixed obscure renaming bug under Windows 95, when editing a file by its
  short file name.

WIN32 (mostly from George Reilly):
- Made window updating faster for NT.  '95 is still slow.
- Fixed digraphs for NT.
- Fixed obscure renaming bug under Windows 95, when editing a file by its
  short file name.
- Use beep by default, use visual bell when 'vb' set.
- Support for foreign keyboards included (again), with fix for CTRL-^ and
  CTRL-@.
- Fixed cursor positioning on exit for NT 4.0 beta.
- Updated emergency exit code, use same function as for Unix.
- Made Makefile.w32 multithreaded.
- When $VIM is not defined, use $HOME.
- Fixed a cursor-positioning bug when outside of termcap mode (e.g., when
  printing "Press RETURN..." after call_shell).
- Changed the initial maps to behave like standard Windows apps, e.g.,
  Ctrl-Home moves to the beginning of the document, not of the screen.

Autocommands:
- Don't show [no write since last change] when executing a shell command from
  an autocommand.
- Always match the file name pattern with the full file name.  A pattern
  like "*src/ *" will also be matched when editing "version.c" with "src" as
  the current directory.  Was different when editing "src/version.c" from one
  directory higher.
- Apply the BufEnter autocommands after recovering.
- ":doautocmd" was not working at all.
- Fixed completion for ":doautocmd".
- Give warning message if nothing executed for ":doautocmd".
- When exiting, after executing the VimLeave autocommands, position the cursor
  at the last screen line again, the autocommands may have moved it.
- When the display can be switched, would ask to hit return for each shell
  command.  Now for shell commands in autocommands the display remains the
  same.

In CTRL-X mode, when 'infercase' is set, don't set 'ignorecase' because of an
uppercase character in the search pattern and 'smartcase' is set.  Make
dictionary completion work on words with an uppercase letter.

Fixed: ":checkpath" listed files that were found already as not found.

When replacing home directory with "~", and it's not followed by anything,
make it "~/".  Looks more like a path name and can't be confused with "~" for
'backupext'.

Fixed expansion of "~" to home directory, was using shell instead of
getenv(HOME).

Changed default for 'backupdir'.  Used to be ".", now it's ".,~" for Unix,
likewise for other systems.  Helps when current directory is not writable (but
the file is) and 'writebackup' is set.

When expanding an option value with <Tab> also replace home directory.

When listing options, check length of number options too, 'maxmapdepth' set to
9999999 would show 999999.  Also account for replacing the home directory when
checking the length of expandable string options.

When doing "ga" on a ^@, which is stored as NL, show values for NUL.

When doing CTRL-D in the command line, don't highlight buffer names that don't
exist.

CTRL-U and CTRL-D now move the cursor when no more scrolling can be done.
CTRL-D does not scroll lines after the end of the file on to the screen.

Don't give "file has changed" warning message for buffers that have their file
name changed with ":file fname".  It's not needed, since these files can't be
normally overwritten.

Made 'infercase' option local to a buffer.

Screen updating for "J" improved a bit.  Was clearing the cursor line instead
of the next line.  Looks smoother now.

When doing CTRL-W CTRL-W when visual mode is active, window was not redrawn.

Added argument to vim_delay(), replaced sleep() with vim_delay().  Renamed
vim_delay() to mch_delay().

For ":sleep" command: Position displayed cursor at actual cursor position.

Quickfix:
- Skip a line if it's obvious that it doesn't match an error format, should
  make parsing the error file a bit faster.
- When Vim is started with "-e errofile" and the error file does not exist,
  would give error message on wrong screen (if switching screens).
  Changed terminal initializations: Would call starttermcap() before reading
  .vimrc, which is wrong.
- Made column for error position one smaller.

Removed restriction on number of screen columns (MAX_COLUMNS).  It was not
used for anything.

Add "version 4.0" to the start of the file generated with ":mkvimrc".
Add a backslash before a '"' for ":mkexrc" and ":mkvimrc".
Don't write 'endofline', 'scroll' and 'ttyfast' with ":mkvimrc" or ":mkexrc".

Included a few patches for the EMX port.

Implemented support for include files for emacs tag files.

Renamed isdir() to mch_isdir(), breakcheck() to mch_breakcheck().

'showmatch': Only show match when visible, not when 'wrap' off and match is
left or right of the screen.

Made 'writedelay' work for more systems, including GUI.  Helps when testing
screen output function.

Added RIGHTLEFT patches from Avner Lottem:
- Put all the reverse insert mode stuff inside RIGHTLEFT.
- Removed TOGGLE_REVINS.
- Added doc/vim_rlh.txt.
- Added 'aleph', 'hkmap' and 'rightleft' options.

Added 'c' flag to 'cpoptions'.  When present, searching is vi-compatible.
When not present, searching is Vim version 3.0 compatible.  The difference is
how often "/abab" matches on "abababababababab".

Fixed bug: "zb" when cursor is in file line 1, followed by "j" many times,
didn't scroll window up when cursor is below the last displayed line.

Renamed "gh" to "zh", "gl" to "zl", added "zs" and "ze" commands.  Reason: "z"
is the scroll-prefix command, not "g".

When using a substitute command in a ":global" command, summarise the number
of substitutions once, instead of giving a message for each line.
Don't call cursupdate() for each line, speeds it up quite a bit and avoids a
redisplay problem.

The message given for the ":global" command about number of extra or deleted
lines was often removed by a redisplay.

Added possibility to use tag file names starting with '.' in 'tags' option.
Made the default "./tags,tags" to first search the tags file in the directory
where the current file is, next the tags file in the current directory.

When opening a new line above/below a comment leader, add a space if it's not
there yet (previously this was only done after a '*').

Don't get a second character for two-character commands when we can tell from
the first character that it's an error.  For example, typing 'Z' while an
operator is pending.

Changed "[I" command to mention included files that include previously listed
matches.

Fixed bug: When 'timeout' off and 'ttimeout' on would still timeout on
mappings.

Fixed bug: Doing "o" in a two-line window moved the cursor an extra line down.
Inserted text could not be removed with "u".

Added <C-Home> and <C-End>, go to start or end of the file.  In Normal mode
and insert mode.  Only for GUI, others don't support these keys.

HAVE_LANGMAP is not defined by default.

Fixed memory leak in fmarks_check_names().

Fixed: Message "freeing N lines" would cause a hit-return message when an
(error) message has been given before it.

Fixed: The ":global" command could set a mark in the jumplist for every
matching line.  Now only set the previous context mark once.  Also makes
":g/pat/s//foo/" run quite a bit faster.

Did some profiling.  Made ":s" a bit faster.  Made cursupdate() and
screen_fill() a bit faster.

Fixed potential problem in prefix_in_list(), was reading past allocated
memory.

Fixed memory leaks for completion in insert mode.

Fixed bug: when 'expandtab' set, and 'smarttab' not set, inserting a tab after
moving the cursor in insert mode could not be repeated.

Fixed: when using 'cindent' while 'autoindent' is off, new indent was not
deleted when typing <Esc>.

Break indenting "=G" when CTRL-C hit.  Report the lines really indented, not
counting the ones that were skipped.

Adjusted errorformat for MS-DOS to include MS Visual C++ error messages.

*/

char		   *Version = "VIM 3.22";
#if !defined(__DATE__) || !defined(__TIME__)
char		   *longVersion = "Vi IMproved 3.22 BETA (1996 Mar 24)";
#else
char		   *longVersion = "Vi IMproved 3.22 BETA (1996 Mar 24, compiled " __DATE__ " " __TIME__ ")";
#endif
