/*
 * File:	vfold.c
 *
 * Synopsis:	folds stdin lines into a tmp file; unfolds pre-folded lines
 *
 * System:	ANSI-C; DOS / unix - needs tempnam()
 *
 * When stdin = any text: saves in a unique file in a tmp area and
 * returns a suitable header.
 * When stdin line1 starts with TAGHEAD: returns previously folded lines
 * as shown within the tag.
 * Mainly for use with vi/vim/elvis fold macros.
 *
 * Copyright (c) 1993 Angelo Haritsis. All rights reserved.
 * C adaptation of the Eric Bowles csh script
 *
 * DISTRIBUTE FREELY, PROVIDED THE ABOVE COPYRIGHT IS KEPT. NO WARRANTIES.
 *
 * $Id: vfold.c,v 1.1 1996/03/30 11:19:06 ariel Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifndef NULL
#define NULL ((char *)0)
#endif

#define TAGHEAD "$$FOLDED:-- "
#define TAGTAIL " --$$"
#define LINESIZE 1024

int
main(void)
{
	char line[LINESIZE];
	char stmp[100];
	char *file, *s;
	FILE *f;
	int lineno;

	if (fgets(line, LINESIZE, stdin) == NULL) /* empty input */
		return (0);
	if (strncmp(line, TAGHEAD, 12) == 0) {	/* it is a tag line: unfold */
		file = &line[12];
		while (*file && *file++ != ' ')
			;
		s = file;
		while (*s && *s++ != ' ')
			;
		if (!*s)	/* not proper tag */
			return (0);
		*(s - 1) = '\0';
		f = fopen(file, "r");
		strncpy(stmp, file, 100);
		if (f == NULL) {
			printf("ACK! Cannot find file %s -- folded text may be lost\n", file);
			return (1);
		}
		while (fgets(line, LINESIZE, f) != NULL)
			fputs(line, stdout);
		fclose(f);
		unlink(stmp);	/* dangerous if undo in vi (cannot redo) */
	} else {		/* fold */
		/* line[strlen(line) - 1] = '\0'; */
		s = getenv("VFOLDDIR");
		file = tempnam(s, "vfold");
		f = fopen(file, "w");
		lineno = 0;
		do {
			lineno++;
			fputs(line, f);
		} while (fgets(line, LINESIZE, stdin) != NULL);
		printf("%s%d %s%s\n", TAGHEAD, lineno, file, TAGTAIL);
		fclose(f);
		free(file);
	}
	return (0);
}
