/* xxd: my hexdump facility. jw
 * changed to word output 2.10.90
 * new indent style, dumb bug inserted and fixed. 3.3.93
 * -c option, mls
 * better option parser, -ps, -l, -s  added, 26.4.94
 * -r badly needs - as input file, added. 1.7.94
 * Per default autoskip over consequtive lines of zeroes, as unix od does.
 * -a shows them too.
 * -i dump as c-style #include "file.h"
 *    if "xxd -i" knows the filename, an 'unsigned char array' is written
 *    in correct c-syntax. added 1.11.95
 *
 * (c) 1990-1995 by Juergen Weigert (jnweiger@informatik.uni-erlangen.de)
 *
 * Distribute freely and credit me, 
 * make money and share with me,
 * lose money and don't ask me.
 */
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>

#define TRY_SEEK 

#define COLS 64
#define LLEN (9 + (5*COLS-1)/2 + 2 + COLS)

#define READ "r"

char hexx[] = "0123456789abcdef";

/* the different hextypes known by this program: */
#define HEX_NORMAL 0
#define HEX_POSTSCRIPT 1
#define HEX_CINCLUDE 2

void
exit_with_usage(pname)
char *pname;
{
  static char use[] = "[-c num] [-ps] [inputfile [outputfile]]";

  fprintf(stderr,
          "Usage: %s [-l length] [-s %sskip] [-a] %s\n", pname,
#ifdef TRY_SEEK
	  "[-]",
#else
	  "",
#endif
	  use);
  fprintf(stderr, "    or\n       %s -r %s\n", pname, use);
  exit(1);
}

int
huntype(fpi, fpo, fperr, pname, cols, hextype)
FILE *fpi, *fpo, *fperr;
char *pname;
int cols, hextype;
{
  int c, ign_garb = 1, n1 = -1, n2, p = cols;
  long have_off = 0, want_off = 0;

  rewind(fpi);

  while ((c = getc(fpi)) != EOF)
    {
      n2 = n1;

      if (c >= '0' && c <= '9')
        n1 = c - '0';
      else if (c >= 'a' && c <= 'f')
        n1 = c - 'a' + 10;
      else if (c >= 'A' && c <= 'F')
        n1 = c - 'A' + 10;
      else 
        {
          n1 = -1;
	  if (ign_garb)
	    continue;
	}

      ign_garb = 0;

      if (!hextype && p >= cols)
        {
	  if (n1 < 0)
	    {
	      p = 0;
	      continue;
	    }
	  want_off = (want_off << 4) | n1;
	  continue;
	}

      if (want_off != have_off)
        {
#ifdef TRY_SEEK
	  c = fseek(fpo, want_off - have_off, 1);
	  if (c >= 0)
	    have_off = want_off;
#endif
	  if (want_off < have_off)
	    {
	      fprintf(fperr, "%s: sorry, cannot seek backwards.\n", pname);
	      return 5;
	    }
	  for (; have_off < want_off; have_off++)
	    putc(0, fpo);
	}

      if (n2 >= 0 && n1 >= 0)
        {
	  putc((n2 << 4) | n1, fpo);
	  fflush(fpo);
	  have_off++;
	  want_off++;
	  n1 = -1;
	  if (++p >= cols)
	    {
	      if (!hextype)
	        want_off = 0;
	      while ((c = getc(fpi)) != '\n' && c != EOF)
	        ;
	      ign_garb = 1;
	    }
	}
    }
#ifdef TRY_SEEK
  fseek(fpo, 0, 2);
#endif
  fclose(fpo);
  fclose(fpi);
  return 0;
}

/* suppress consecutive lines of zeroes. */
void
outline(l, nz)
char *l;
int nz;
{
  static char z[LLEN+1];
  static int zero_seen = 0;

  if (!nz && zero_seen == 1)
    strcpy(z, l);

  if (nz || !zero_seen++)
    {
      if (nz)
	{
	  if (zero_seen == 2)
	    puts(z);
	  if (zero_seen > 2)
	    puts("*");
	  zero_seen = 0;
	}
      puts(l);
    }
}

int
main(argc, argv)
int argc;
char *argv[];
{
  FILE *fp, *fpo;
  int c, e, p = 0, skip = 0, negskip = 0, hextype, revert = 0, autoskip = 1;
  long length = 0, n = 0;
  char l[LLEN+1];
  int cols = 0, nonzero = 0;
  char *pname, *pp;
  hextype = HEX_NORMAL;

  pname = argv[0];
  for (pp = pname; *pp; )
    if (*pp++ == '/')
      pname = pp;

  while (argc >= 2)
    {
      if (!strncmp(argv[1], "-c", 2))
	{
	  if (argv[1][2] && strncmp("ols", argv[1] + 2, 3))
	    cols = strtol(argv[1] + 2, NULL, 0);
	  else
	    {
	      if (!argv[2])
	        exit_with_usage(pname);
	      cols = strtol(argv[2], NULL, 0);
	      argv++;
	      argc--;
	    }
	  argv++;
	  argc--;
	}
      else if (!strncmp(argv[1], "-a", 2))
        {
	  autoskip = 0;
	  argv++;
	  argc--;
	}
      else if (!strncmp(argv[1], "-p", 2))
	{
	  hextype = HEX_POSTSCRIPT;
	  argv++;
	  argc--;
	}
      else if (!strncmp(argv[1], "-i", 2))
	{
	  hextype = HEX_CINCLUDE;
	  argv++;
	  argc--;
	}
      else if (!strncmp(argv[1], "-s", 2))
	{
	  if (argv[1][2] && strncmp("kip", argv[1] +2, 3))
	    {
#ifdef TRY_SEEK
	      if (argv[1][2] == '-')
	        negskip++;
#endif
	      skip = strtol(argv[1] + 2 + negskip, (char **)NULL, 0);
	    }
	  else
	    {
	      if (!argv[2])
	        exit_with_usage(pname);
#ifdef TRY_SEEK
	      if (argv[2][0] == '-')
	        negskip++;
#endif
	      skip = strtol(argv[2] + negskip, (char **)NULL, 0);
	      argv++;
	      argc--;
	    }
	  argv++;
	  argc--;
	}
      else if (!strncmp(argv[1], "-l", 2))
	{
	  if (argv[1][2] && strncmp("en", argv[1] + 2, 2))
	    length = strtol(argv[1] + 2, (char **)NULL, 0);
	  else
	    {
	      if (!argv[2])
	        exit_with_usage(pname);
	      length = strtol(argv[2], (char **)NULL, 0);
	      argv++;
	      argc--;
	    }
	  argv++;
	  argc--;
	}
      else if (!strcmp(argv[1], "-r"))
        {
	  revert++;
	  argv++;
	  argc--;
	}
      else if (!strcmp(argv[1], "--"))	/* end of options */
        {
	  argv++;
	  argc--;
	  break;
	}
      else if (argv[1][0] == '-' && argv[1][1])	/* unknown option */
        exit_with_usage(pname);
      else 
        break;				/* not an option */
    }

  if (!cols)
    switch (hextype)
      {
      case HEX_POSTSCRIPT: 	cols = 30; break;
      case HEX_CINCLUDE:	cols = 12; break;
      case HEX_NORMAL:
      default:			cols = 16; break;
      }

  if (cols < 1 || (!hextype && (cols > COLS)))
    {
      fprintf(stderr, "%s: invalid number of columns.\n", pname);
      exit(1);
    }

  if (argc > 3) 
    exit_with_usage(pname);

  if (argc == 1 || (argv[1][0] == '-' && !argv[1][1]))
    fp = stdin;
  else
    {
      if ((fp = fopen(argv[1], READ)) == NULL)
	{
	  fprintf(stderr,"%s: ", pname);
	  perror(argv[1]);
	  return 2;
	}
    }
  
  if (argc < 3 || (argv[2][0] == '-' && !argv[2][1]))
    fpo = stdout;
  else
    {
      int fd;

      if (((fd = open(argv[2], O_WRONLY | O_CREAT, 0666)) < 0) ||
          (fpo = fdopen(fd, "w")) == NULL)
	{
	  fprintf(stderr, "%s: ", pname);
	  perror(argv[2]);
	  return 3;
	}
      rewind(fpo);
    }
  
  if (revert)
    {
      if (hextype && (hextype != HEX_POSTSCRIPT))
        {
          fprintf(stderr, "%s: sorry, cannot revert this type of hexdump\n", pname);
	  return -1;
	}
      return huntype(fp, fpo, stderr, pname, cols, hextype);
    }

  if (skip)
    {
#ifdef TRY_SEEK
      e = fseek(fp, negskip ? -skip : skip, negskip ? 2 : 0);
      if (e < 0 && negskip)
        {
	  fprintf(stderr, "%s: sorry cannot seek.\n", pname);
	  return 4;
	}
      if (e >= 0)
        skip = ftell(fp);
      else
#endif
	{
	  e = skip;
	  while (e--)
	    getc(fp); 
	}
    }

  if (hextype == HEX_CINCLUDE)
    {
      if (fp != stdin)
	{
	  fprintf(fpo, "unsigned char ");
	  for (e = 0; (c = argv[1][e]); e++)
	    putc(isalnum(c) ? c : '_', fpo);
	  fputs("[] = {\n", fpo);
	}

      p = 0;
      while ((c = getc(fp)) != EOF)
	{
	  if (length && p >= length)
	    break;
	  fprintf(fpo, "%s0x%02x", (p % cols) ? ", " : ",\n  "+2*!p,  c);
	  p++;
	}

      fputs("\n};\n"+3*(fp == stdin), fpo);

      if (fp != stdin)
	{
	  fprintf(fpo, "unsigned int ");
	  for (e = 0; (c = argv[1][e]); e++)
	    putc(isalnum(c) ? c : '_', fpo);
	  fprintf(fpo, "_len = %d;\n", p);
	}

      fclose(fp);
      fclose(fpo);
      return 0;
    }

  if (hextype == HEX_POSTSCRIPT)
    {
      p = cols;
      while ((e = getc(fp)) != EOF)
	{
	  if (length && n >= length)
	    break;
          putchar(hexx[(e >> 4) & 0xf]);
          putchar(hexx[(e     ) & 0xf]);
	  n++;
	  if (!--p)
	    {
	      putchar('\n');
	      p = cols;
	    }
	}
      if (p)
	putchar('\n');
      fclose(fp);
      fclose(fpo);
      return 0;
    }

  while ((e = getc(fp)) != EOF)
    {
      if (length && n >= length)
	break;
      if (p == 0)
	{
	  sprintf(l, "%07x:", n + skip);
	  for (c = 8; c < LLEN; l[c++] = ' ');
	}
      l[9 + (5 * p) / 2] = hexx[(e >> 4) & 0xf];
      l[10 + (5 * p) / 2] = hexx[ e       & 0xf];
      l[11 + (5 * cols - 1) / 2 + p] = (e > 31 && e < 127) ? e : '.';
      if (e) 
        nonzero++;
      if (++p == cols)
	{
	  l[11 + (5 * cols - 1) / 2 + p] = '\0';
	  outline(l, autoskip ? nonzero : 1);
	  nonzero = 0;
	  n += cols;
	  p = 0;
	}
    }
  if (p)
    {
      l[11 + (5 * cols - 1) / 2 + p] = 0;
      outline(l, autoskip ? nonzero : 1);
    }
  fclose(fp);
  fclose(fpo);
  return 0;
}
